@@ -14,11 +14,13 @@ Submodule listing accepted keithley TSP commands.
 
 """
 
+import itertools
+
 WARNING = """Warning: PROPERTIES, CONSTANTS and FUNCTIONS must
 not contain elements from CLASSES. E.g., once a property is created,
 it cannot be used as a class anymore"""
 
-CONSTANTS = [
+CONSTANTS = (
     'CAPACITY',
     'ARMED_EVENT_ID',
     'MEASURE_COMPLETE_EVENT_ID',
@@ -49,9 +51,9 @@ CONSTANTS = [
     'AUTORANGE_FOLLOW_LIMIT',
     'ON',
     'OFF',
-]
+)
 
-FUNCTIONS = [
+FUNCTIONS = (
     'beep',
     'bitand',
     'bitor',
@@ -247,9 +249,9 @@ FUNCTIONS = [
     'iv',
     'r',
     'p',
-]
+)
 
-PROPERTIES = [
+PROPERTIES = (
     'enable',
     'appendmode',
     'basetimestamp',
@@ -437,9 +439,9 @@ PROPERTIES = [
     'event',
     'ntr',
     'ptr',
-]
+)
 
-CLASSES = [
+CLASSES = (
     'arm',
     'beeper',
     'bit',
@@ -512,9 +514,9 @@ CLASSES = [
     'user',
     'userstring',
     'voltage_limit',
-]
+)
 
-ALL_CMDS = [
+ALL_CMDS_WITH_PLACEHOLDERS = (
     'InitiatePulseTest',
     'InitiatePulseTestDual',
     'QueryPulseConfig',
@@ -530,48 +532,27 @@ ALL_CMDS = [
     'bit.setfield',
     'bit.test',
     'bit.toggle',
-    'nvbuffer1.appendmode',
-    'nvbuffer1.basetimestamp',
-    'nvbuffer1.cachemode',
-    'nvbuffer1.capacity',
-    'nvbuffer1.clear',
-    'nvbuffer1.clearcache',
-    'nvbuffer1.collectsourcevalues',
-    'nvbuffer1.collecttimestamps',
-    'nvbuffer1.fillcount',
-    'nvbuffer1.fillmode',
-    'nvbuffer1.measurefunctions',
-    'nvbuffer1.measureranges',
-    'nvbuffer1.n',
-    'nvbuffer1.readings',
-    'nvbuffer1.sourcefunctions',
-    'nvbuffer1.sourceoutputstates',
-    'nvbuffer1.sourceranges',
-    'nvbuffer1.sourcevalues',
-    'nvbuffer1.statuses',
-    'nvbuffer1.timestampresolution',
-    'nvbuffer1.timestamps',
-    'nvbuffer2.appendmode',
-    'nvbuffer2.basetimestamp',
-    'nvbuffer2.cachemode',
-    'nvbuffer2.capacity',
-    'nvbuffer2.clear',
-    'nvbuffer2.clearcache',
-    'nvbuffer2.collectsourcevalues',
-    'nvbuffer2.collecttimestamps',
-    'nvbuffer2.fillcount',
-    'nvbuffer2.fillmode',
-    'nvbuffer2.measurefunctions',
-    'nvbuffer2.measureranges',
-    'nvbuffer2.n',
-    'nvbuffer2.readings',
-    'nvbuffer2.sourcefunctions',
-    'nvbuffer2.sourceoutputstates',
-    'nvbuffer2.sourceranges',
-    'nvbuffer2.sourcevalues',
-    'nvbuffer2.statuses',
-    'nvbuffer2.timestampresolution',
-    'nvbuffer2.timestamps',
+    'nvbuffer#.appendmode',
+    'nvbuffer#.basetimestamp',
+    'nvbuffer#.cachemode',
+    'nvbuffer#.capacity',
+    'nvbuffer#.clear',
+    'nvbuffer#.clearcache',
+    'nvbuffer#.collectsourcevalues',
+    'nvbuffer#.collecttimestamps',
+    'nvbuffer#.fillcount',
+    'nvbuffer#.fillmode',
+    'nvbuffer#.measurefunctions',
+    'nvbuffer#.measureranges',
+    'nvbuffer#.n',
+    'nvbuffer#.readings',
+    'nvbuffer#.sourcefunctions',
+    'nvbuffer#.sourceoutputstates',
+    'nvbuffer#.sourceranges',
+    'nvbuffer#.sourcevalues',
+    'nvbuffer#.statuses',
+    'nvbuffer#.timestampresolution',
+    'nvbuffer#.timestamps',
     'dataqueue.CAPACITY',
     'dataqueue.add',
     'dataqueue.clear',
@@ -610,12 +591,9 @@ ALL_CMDS = [
     'display.sendkey',
     'display.setcursor',
     'display.settext',
-    'display.smua.digits',
-    'display.smua.limit.func',
-    'display.smua.measure.func',
-    'display.smub.digits',
-    'display.smub.limit.func',
-    'display.smub.measure.func',
+    'display.smu£.digits',
+    'display.smu£.limit.func',
+    'display.smu£.measure.func',
     'display.trigger.EVENT_ID',
     'display.trigger.clear',
     'display.trigger.overrun',
@@ -746,225 +724,105 @@ ALL_CMDS = [
     'setup.poweron',
     'setup.recall',
     'setup.save',
-    'smua.abort',
-    'smua.buffer.getstats',
-    'smua.buffer.recalculatestats',
-    'smua.cal.adjustdate',
-    'smua.cal.date',
-    'smua.cal.due',
-    'smua.cal.lock',
-    'smua.cal.password',
-    'smua.cal.polarity',
-    'smua.cal.restore',
-    'smua.cal.save',
-    'smua.cal.state',
-    'smua.cal.unlock',
-    'smua.contact.calibratehi',
-    'smua.contact.calibratelo',
-    'smua.contact.check',
-    'smua.contact.r',
-    'smua.contact.speed',
-    'smua.contact.threshold',
-    'smua.makebuffer',
-    'smua.measure.v',
-    'smua.measure.i',
-    'smua.measure.analogfilter',
-    'smua.measure.autorangev',
-    'smua.measure.autorangei',
-    'smua.measure.autozero',
-    'smua.measure.calibratev',
-    'smua.measure.calibratei',
-    'smua.measure.count',
-    'smua.measure.delay',
-    'smua.measure.delayfactor',
-    'smua.measure.filter.count',
-    'smua.measure.filter.enable',
-    'smua.measure.filter.type',
-    'smua.measure.highcrangedelayfactor',
-    'smua.measure.interval',
-    'smua.measure.lowrangev',
-    'smua.measure.lowrangei',
-    'smua.measure.nplc',
-    'smua.measure.overlappedv',
-    'smua.measure.overlappedi',
-    'smua.measure.overlappedr',
-    'smua.measure.overlappedp',
-    'smua.measure.rangev',
-    'smua.measure.rangei',
-    'smua.measure.rel.enablev',
-    'smua.measure.rel.enablei',
-    'smua.measure.rel.enabler',
-    'smua.measure.rel.enablep',
-    'smua.measure.rel.levelv',
-    'smua.measure.rel.leveli',
-    'smua.measure.rel.levelr',
-    'smua.measure.rel.levelp',
-    'smua.measurevandstep',
-    'smua.measureiandstep',
-    'smua.measurerandstep',
-    'smua.measurepandstep',
-    'smua.nvbufferY',
-    'smua.nvbufferY.readings[N]',
-    'smua.reset',
-    'smua.savebuffer',
-    'smua.sense',
-    'smua.source.autorangei',
-    'smua.source.autorangev',
-    'smua.source.calibratei',
-    'smua.source.calibratev',
-    'smua.source.compliance',
-    'smua.source.delay',
-    'smua.source.func',
-    'smua.source.highc',
-    'smua.source.leveli',
-    'smua.source.levelv',
-    'smua.source.limiti',
-    'smua.source.limitv',
-    'smua.source.lowrangei',
-    'smua.source.lowrangev',
-    'smua.source.offfunc',
-    'smua.source.offlimiti',
-    'smua.source.offlimitv',
-    'smua.source.offmode',
-    'smua.source.output',
-    'smua.source.outputenableaction',
-    'smua.source.rangev',
-    'smua.source.rangei',
-    'smua.source.settling',
-    'smua.source.sink',
-    'smua.trigger.ARMED_EVENT_ID',
-    'smua.trigger.IDLE_EVENT_ID',
-    'smua.trigger.MEASURE_COMPLETE_EVENT_ID',
-    'smua.trigger.PULSE_COMPLETE_EVENT_ID',
-    'smua.trigger.SOURCE_COMPLETE_EVENT_ID',
-    'smua.trigger.SWEEPING_EVENT_ID',
-    'smua.trigger.SWEEP_COMPLETE_EVENT_ID',
-    'smua.trigger.arm.count',
-    'smua.trigger.arm.set',
-    'smua.trigger.arm.stimulus',
-    'smua.trigger.autoclear',
-    'smua.trigger.count',
-    'smua.trigger.endpulse.action',
-    'smua.trigger.endpulse.set',
-    'smua.trigger.endpulse.stimulus',
-    'smua.trigger.endsweep.action',
-    'smua.trigger.initiate',
-    'smua.trigger.measure.i',
-    'smua.trigger.measure.v',
-    'smua.trigger.measure.r',
-    'smua.trigger.measure.p',
-    'smua.trigger.measure.action',
-    'smua.trigger.measure.set',
-    'smua.trigger.measure.stimulus',
-    'smua.trigger.source.action',
-    'smua.trigger.source.limiti',
-    'smua.trigger.source.limitv',
-    'smua.trigger.source.lineari',
-    'smua.trigger.source.linearv',
-    'smua.trigger.source.listi',
-    'smua.trigger.source.listv',
-    'smua.trigger.source.logY',
-    'smua.trigger.source.logi',
-    'smua.trigger.source.logv',
-    'smua.trigger.source.set',
-    'smua.trigger.source.stimulus',
-    'smub.abort',
-    'smub.buffer.getstats',
-    'smub.buffer.recalculatestats',
-    'smub.cal.adjustdate',
-    'smub.cal.date',
-    'smub.cal.due',
-    'smub.cal.lock',
-    'smub.cal.password',
-    'smub.cal.polarity',
-    'smub.cal.restore',
-    'smub.cal.save',
-    'smub.cal.state',
-    'smub.cal.unlock',
-    'smub.contact.calibratehi',
-    'smub.contact.calibratelo',
-    'smub.contact.check',
-    'smub.contact.r',
-    'smub.contact.speed',
-    'smub.contact.threshold',
-    'smub.makebuffer',
-    'smub.measure.Y',
-    'smub.measure.analogfilter',
-    'smub.measure.autorangeY',
-    'smub.measure.autozero',
-    'smub.measure.calibrateY',
-    'smub.measure.count',
-    'smub.measure.delay',
-    'smub.measure.delayfactor',
-    'smub.measure.filter.count',
-    'smub.measure.filter.enable',
-    'smub.measure.filter.type',
-    'smub.measure.highcrangedelayfactor',
-    'smub.measure.interval',
-    'smub.measure.lowrangeY',
-    'smub.measure.nplc',
-    'smub.measure.overlappedY',
-    'smub.measure.rangeY',
-    'smub.measure.rel.enableY',
-    'smub.measure.rel.levelY',
-    'smub.measureYandstep',
-    'smub.nvbufferY',
-    'smub.nvbufferY.readings[N]',
-    'smub.reset',
-    'smub.savebuffer',
-    'smub.sense',
-    'smub.source.autorangeY',
-    'smub.source.calibrateY',
-    'smub.source.compliance',
-    'smub.source.delay',
-    'smub.source.func',
-    'smub.source.highc',
-    'smub.source.levelY',
-    'smub.source.limitY',
-    'smub.source.lowrangeY',
-    'smub.source.offfunc',
-    'smub.source.offlimitY',
-    'smub.source.offmode',
-    'smub.source.output',
-    'smub.source.outputenableaction',
-    'smub.source.rangeY',
-    'smub.source.settling',
-    'smub.source.sink',
-    'smub.trigger.ARMED_EVENT_ID',
-    'smub.trigger.IDLE_EVENT_ID',
-    'smub.trigger.MEASURE_COMPLETE_EVENT_ID',
-    'smub.trigger.PULSE_COMPLETE_EVENT_ID',
-    'smub.trigger.SOURCE_COMPLETE_EVENT_ID',
-    'smub.trigger.SWEEPING_EVENT_ID',
-    'smub.trigger.SWEEP_COMPLETE_EVENT_ID',
-    'smub.trigger.arm.count',
-    'smub.trigger.arm.set',
-    'smub.trigger.arm.stimulus',
-    'smub.trigger.autoclear',
-    'smub.trigger.count',
-    'smub.trigger.endpulse.action',
-    'smub.trigger.endpulse.set',
-    'smub.trigger.endpulse.stimulus',
-    'smub.trigger.endsweep.action',
-    'smub.trigger.initiate',
-    'smub.trigger.measure.Y',
-    'smub.trigger.measure.action',
-    'smub.trigger.measure.set',
-    'smub.trigger.measure.stimulus',
-    'smub.trigger.source.action',
-    'smub.trigger.source.limitY',
-    'smub.trigger.source.linearY',
-    'smub.trigger.source.listY',
-    'smub.trigger.source.logY',
-    'smub.trigger.source.set',
-    'smub.trigger.source.stimulus',
+    'smu£.abort',
+    'smu£.buffer.getstats',
+    'smu£.buffer.recalculatestats',
+    'smu£.cal.adjustdate',
+    'smu£.cal.date',
+    'smu£.cal.due',
+    'smu£.cal.lock',
+    'smu£.cal.password',
+    'smu£.cal.polarity',
+    'smu£.cal.restore',
+    'smu£.cal.save',
+    'smu£.cal.state',
+    'smu£.cal.unlock',
+    'smu£.contact.calibratehi',
+    'smu£.contact.calibratelo',
+    'smu£.contact.check',
+    'smu£.contact.r',
+    'smu£.contact.speed',
+    'smu£.contact.threshold',
+    'smu£.makebuffer',
+    'smu£.measure.%',
+    'smu£.measure.analogfilter',
+    'smu£.measure.autorange@',
+    'smu£.measure.autozero',
+    'smu£.measure.calibrate@',
+    'smu£.measure.count',
+    'smu£.measure.delay',
+    'smu£.measure.delayfactor',
+    'smu£.measure.filter.count',
+    'smu£.measure.filter.enable',
+    'smu£.measure.filter.type',
+    'smu£.measure.highcrangedelayfactor',
+    'smu£.measure.interval',
+    'smu£.measure.lowrange@',
+    'smu£.measure.nplc',
+    'smu£.measure.overlapped%',
+    'smu£.measure.range@',
+    'smu£.measure.rel.enable%',
+    'smu£.measure.rel.level%',
+    'smu£.measure%andstep',
+    'smu£.nvbuffer#',
+    'smu£.nvbuffer#.readings[N]',
+    'smu£.reset',
+    'smu£.savebuffer',
+    'smu£.sense',
+    'smu£.SENSE_LOCAL',
+    'smu£.SENSE_REMOTE',
+    'smu£.SENSE_CALA',
+    'smu£.source.autorange@',
+    'smu£.source.calibrate@',
+    'smu£.source.compliance',
+    'smu£.source.delay',
+    'smu£.source.func',
+    'smu£.source.highc',
+    'smu£.source.level@',
+    'smu£.source.limit@',
+    'smu£.source.lowrange@',
+    'smu£.source.offfunc',
+    'smu£.source.offlimit@',
+    'smu£.source.offmode',
+    'smu£.source.output',
+    'smu£.source.outputenableaction',
+    'smu£.source.range@',
+    'smu£.source.settling',
+    'smu£.source.sink',
+    'smu£.trigger.ARMED_EVENT_ID',
+    'smu£.trigger.IDLE_EVENT_ID',
+    'smu£.trigger.MEASURE_COMPLETE_EVENT_ID',
+    'smu£.trigger.PULSE_COMPLETE_EVENT_ID',
+    'smu£.trigger.SOURCE_COMPLETE_EVENT_ID',
+    'smu£.trigger.SWEEP_COMPLETE_EVENT_ID',
+    'smu£.trigger.SWEEPING_EVENT_ID',
+    'smu£.trigger.arm.count',
+    'smu£.trigger.arm.set',
+    'smu£.trigger.arm.stimulus',
+    'smu£.trigger.autoclear',
+    'smu£.trigger.count',
+    'smu£.trigger.endpulse.action',
+    'smu£.trigger.endpulse.set',
+    'smu£.trigger.endpulse.stimulus',
+    'smu£.trigger.endsweep.action',
+    'smu£.trigger.initiate',
+    'smu£.trigger.measure.',
+    'smu£.trigger.measure.action',
+    'smu£.trigger.measure.set',
+    'smu£.trigger.measure.stimulus',
+    'smu£.trigger.source.action',
+    'smu£.trigger.source.limit@',
+    'smu£.trigger.source.linear@',
+    'smu£.trigger.source.list@',
+    'smu£.trigger.source.log@',
+    'smu£.trigger.source.set',
+    'smu£.trigger.source.stimulus',
     'status.condition',
     'status.measurement.*',
     'status.measurement.buffer_available.*',
     'status.measurement.current_limit.*',
     'status.measurement.instrument.*',
-    'status.measurement.instrument.smua.*',
-    'status.measurement.instrument.smub.*',
+    'status.measurement.instrument.smu£.*',
     'status.measurement.reading_overflow.*',
     'status.measurement.voltage_limit.*',
     'status.node_enable',
@@ -975,10 +833,8 @@ ALL_CMDS = [
     'status.operation.instrument.digio.trigger_overrun.*',
     'status.operation.instrument.lan.*',
     'status.operation.instrument.lan.trigger_overrun.*',
-    'status.operation.instrument.smua.*',
-    'status.operation.instrument.smua.trigger_overrrun.*',
-    'status.operation.instrument.smub.*',
-    'status.operation.instrument.smub.trigger_overrrun.*',
+    'status.operation.instrument.smu£.*',
+    'status.operation.instrument.smu£.trigger_overrrun.*',
     'status.operation.instrument.trigger_blender.*',
     'status.operation.instrument.trigger_blender.trigger_overrun.*',
     'status.operation.instrument.trigger_timer.*',
@@ -993,8 +849,7 @@ ALL_CMDS = [
     'status.questionable.*',
     'status.questionable.calibration.*',
     'status.questionable.instrument.*',
-    'status.questionable.instrument.smua.*',
-    'status.questionable.instrument.smub.*',
+    'status.questionable.instrument.smu£.*',
     'status.questionable.over_temperature.*',
     'status.questionable.unstable_output.*',
     'status.request_enable',
@@ -1067,9 +922,9 @@ ALL_CMDS = [
     'userstring.delete',
     'userstring.get',
     'waitcomplete',
-]
+)
 
-PROPERTY_LISTS = [
+PROPERTY_LISTS = (
     'trigger.blender[1].stimulus',
     'trigger.blender[2].stimulus',
     'smua.nvbuffer1.readings',
@@ -1077,7 +932,15 @@ PROPERTY_LISTS = [
     'smub.nvbuffer1.readings',
     'smub.nvbuffer2.readings',
     'readings',
-]
+)
+
+PLACEHOLDERS = {
+    '*': ['condition', 'enable', 'event', 'ntr', 'ptr'],
+    '#': ['1', '2'],
+    '£': ['a', 'b'],
+    '@': ['v', 'i'],
+    '%': ['v', 'i', 'r', 'p'],
+}
 
 properties = set(PROPERTIES)
 functions = set(FUNCTIONS)
@@ -1090,3 +953,19 @@ check2 = bool(constants & classes)
 
 if check0 or check1 or check2:
     raise RuntimeError(WARNING)
+
+ALL_CMDS = list(ALL_CMDS_WITH_PLACEHOLDERS)
+
+for cmd in ALL_CMDS_WITH_PLACEHOLDERS:
+    placeholders = list(filter(lambda x: x in PLACEHOLDERS, cmd))
+    if len(placeholders) > 0:
+        ALL_CMDS.remove(cmd)
+        values = [PLACEHOLDERS[p] for p in placeholders]
+        new_cmds = []
+        for value_tuple in itertools.product(*values):
+            new_cmd = cmd
+            for p, v in zip(placeholders, value_tuple):
+                new_cmd = new_cmd.replace(p, v, 1)
+            new_cmds.append(new_cmd)
+
+        ALL_CMDS += new_cmds
