@@ -9,7 +9,7 @@ Core driver with the low level functions.
 """
 
 # system imports
-import visa
+import pyvisa
 import logging
 import threading
 import numpy as np
@@ -22,7 +22,7 @@ from keithley2600.keithley_doc import (CONSTANTS, FUNCTIONS, PROPERTIES,
                                        CLASSES, PROPERTY_LISTS, ALL_CMDS)
 from keithley2600.result_table import FETResultTable
 
-__version__ = 'v1.4.0'
+__version__ = 'v1.4.1'
 
 logger = logging.getLogger(__name__)
 
@@ -361,7 +361,7 @@ class Keithley2600Base(MagicClass):
         self.raise_keithley_errors = raise_keithley_errors
 
         # open visa resource manager with selected library / backend
-        self.rm = visa.ResourceManager(self.visa_library)
+        self.rm = pyvisa.ResourceManager(self.visa_library)
         # connect to keithley
         self.connect(**kwargs)
 
